package at.ac.tuwien.sepm.assignment.individual.service;

import at.ac.tuwien.sepm.assignment.individual.entity.Horse;
import at.ac.tuwien.sepm.assignment.individual.exception.NotFoundException;
import at.ac.tuwien.sepm.assignment.individual.util.ValidationException;
import org.springframework.dao.DataAccessException;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

public interface HorseService {
    /**
     * @param id of the horse to find.
     * @return the horse with the specified id.
     * @throws RuntimeException  will be thrown if something goes wrong during data processing.
     * @throws NotFoundException will be thrown if the horse could not be found in the system.
     */
    Horse findOneById(Long id);

    /**
     * @param horse to add.
     * @return the new horse.
     * @throws ValidationException will be thrown if something goes wrong during verification.
     * @throws DataAccessException will be thrown if the horse could not be saved in the database.
     */
    Horse addHorse(Horse horse);

    /**
     * @param horse that specifies the new horse values alongside with the id of the horse to update
     * @return the updated horse
     * @throws ValidationException will be thrown if something goes wrong during verification.
     * @throws DataAccessException will be thrown if the horse could not be saved in the database.
     * @throws IOException         will be thrown if the old horse image could not be deleted
     */
    Horse updateHorse(Horse horse) throws ValidationException, DataAccessException, IOException;

    /**
     * @param id of the horse to delete
     * @throws NotFoundException   will be thrown if the horse could not be found in the system
     * @throws DataAccessException will be thrown if the horse could not be deleted from the database
     * @throws IOException         will be thrown if the horse image could not be deleted
     */
    void deleteHorse(Long id) throws NotFoundException, DataAccessException, IOException;

    /**
     * @param img image to upload
     * @throws IOException         will be thrown if something goes wrong with saving the file
     * @throws ValidationException will be thrown if the file is in the incorrect format
     */
    void saveImage(MultipartFile img) throws IOException, ValidationException;
}
